#!/bin/tcsh

###########################################################################
## 09/18/2017 Justin Rajendra
## a script to install and configure afni on macOS

## current versions of programs
set R_ver = "3.5.1"
set XQ_ver = "2.7.11"
set afni_ver = "macos_10.12_local"
set afni_web = "https://afni.nimh.nih.gov/pub/dist/bin"
set afni_link = "${afni_web}/${afni_ver}/@update.afni.binaries"
set loud = 0
set localPkg = ""

###########################################################################
## parse arguments for those fools running on the command line
set prog = `basename $0`
if ( $#argv > 0 ) then

    set narg = 1
    @  amax = $#argv - 0
    while ( $narg <= $amax )
        if ( "$argv[$narg]" == "-local_package" ) then
            @ narg ++
            set localPkg = `echo $argv[$narg]`
        else if ( "$argv[$narg]" == "-loud" ) then
            set loud = 1
        else if ( "$argv[$narg]" == "-help" || "$argv[$narg]" == "-h" ) then
            goto SHOW_HELP
        else
            goto SHOW_HELP
        endif
        @ narg ++
    end
endif

###########################################################################
## greetings
echo
echo "******************************************************"
echo ; echo Welcome to the afni installation for  macOS !
echo Press Enter to accept the default selection shown in "[  ]".
echo You may need to type in your administrator password for some packages.
echo If you do not have '"sudo"' powers, this installation may fail. ; echo
echo This will download and install the following if necessary:
echo "    1. Xcode command line tools"
echo "    2. Xquartz"
echo "    3. R"
echo "    4. AFNI" ; echo
echo "******************************************************"
echo

###########################################################################
## check if afni is installed already
set afni_bin = `where afni`
if ( "$afni_bin" != "" ) then
    echo "AFNI is already installed!"
    if ( $loud ) then
        say afni is already installed!
    endif
    echo "Enter 'yes' to continue anyway."
    echo "or enter anything else to exit"
    set afni_try_update = $<
    echo
    if ( $afni_try_update != "yes" ) then
        echo "Thank you and goodbye!" ; echo
        if ( $loud ) then
            say "Thank you and goodbye!"
        endif
        exit 0
    endif
endif   ## end check if afni is there

###########################################################################
## get target directory for afni
echo Enter the target directory for afni installation:
echo "[~/abin]"
set bindir = $<
if ( $bindir == "" ) then
    set bindir = "~/abin"
endif

# ## check if you have write permissions
# if ( ! -w "${bindir}" ) then
#     echo ; echo "ERROR: cannot write to ${bindir}!"
#     echo "Do you have write permissions for ${bindir}?" ; echo
#     exit 1
# endif

###########################################################################
## make a temp folder for the installation downloads
set inst_dir = "$HOME/afni_install_temp_delete_me"
if ( -d "${inst_dir}" ) then
    \rm -r $inst_dir
endif
\mkdir -p $inst_dir
cd $inst_dir

## prepare for interuption
onintr CLEAN_BAD

###########################################################################
## xcode command line tools
set XC_ins = `xcode-select --install |& grep -c installed`
if ( $XC_ins == 0 ) then
    echo ; echo Download and install xcode command line tools with GUI.
    echo Click Install and follow the prompts.
    echo Return to the terminal when complete. ; echo
    xcode-select --install
    echo ; echo Press any key to continue AFTER the command line tools finishes.
    set XC_cont = $<
else
    echo "Xcode command line tools already installed. Skipping." ; echo
    if ( $loud ) then
        say "X code command line tools already installed. Skipping."
    endif
endif

###########################################################################
## Xquartz
set XQ_bin = `where Xquartz`
if ( $XQ_bin == "" ) then
    set XQ_inst = 1
else
    echo ; echo "Xquartz is installed at ${XQ_bin}"
    if ( $loud ) then
        say "X quartz is already installed!"
    endif
    echo "Install anyway?" ; echo
    echo "Enter 'yes' to reinstall Xquartz."
    echo "or enter anything else to continue"
    set XQ_check = $<
    if ( $XQ_check == "yes" ) then
        set XQ_inst = 1
    else
        set XQ_inst = 0
    endif
endif   ## end exists check

if ( $XQ_inst == 1 ) then
    echo ; echo Download and install Xquartz.
    echo You may need your password. ; echo
    if ( $loud ) then
        say "Installing X quartz"
    endif
    ## if already mounted, force eject
    if ( -d "/Volumes/XQuartz-${XQ_ver}" ) then
        hdiutil unmount -force "/Volumes/XQuartz-${XQ_ver}"
    endif

    ## download with redirects and make sure it finishes
    curl -LO "https://dl.bintray.com/xquartz/downloads/XQuartz-${XQ_ver}.dmg"
    if ( -f "XQuartz-${XQ_ver}.dmg" ) then
        hdiutil attach XQuartz-${XQ_ver}.dmg
        if ( $loud ) then
            say Please enter password below
        endif
        echo ; echo ; echo
        sudo installer -verbose -pkg /Volumes/XQuartz-${XQ_ver}/XQuartz.pkg -target /

        ## check to see if it failed
        if ( $status != 0 ) then
            echo ; echo "XQuartz installation failed. Do you have 'sudo' powers?"
            echo "Please install XQuartz manually and rerun this program."
            echo ; echo "https://www.xquartz.org/" ; echo
            hdiutil unmount "/Volumes/XQuartz-${XQ_ver}"
            \rm XQuartz-${XQ_ver}.dmg
            goto CLEAN_BAD
        endif
        hdiutil unmount "/Volumes/XQuartz-${XQ_ver}"
        \rm XQuartz-${XQ_ver}.dmg
    else
        echo ; echo "Download failed! Please install Xquartz manually."
        echo ; echo "https://www.xquartz.org" ; echo
        echo "Then rerun this program."
        goto CLEAN_BAD
    endif
else
    echo ; echo Xquartz is installed at $XQ_bin
    echo Skipping Xquartz installation. ; echo
endif   ## end Xquartz check

## click through inactive windows
defaults write org.macosforge.xquartz.X11 wm_click_through -bool true

###########################################################################
## R installation
set R_bin = `where R`
if ( "$R_bin" == "" ) then
    set R_inst = 1
else
    echo ; echo R is installed at $R_bin
    if ( $loud ) then
        say "R is already installed!"
    endif
    echo "Install anyway?" ; echo
    echo "Enter 'yes' to reinstall R."
    echo "or enter anything else to continue"
    set R_check = $<
    if ( $R_check == "yes" ) then
        set R_inst = 1
    else
        set R_inst = 0
    endif
endif   ## end exists check

if ( $R_inst ) then
    echo ; echo Download and install R.
    echo You may need your password. ; echo

    ## download and check and install
    curl -LO "https://cran.r-project.org/bin/macosx/R-${R_ver}.pkg"
    if ( -f "R-${R_ver}.pkg" ) then
        if ( $loud ) then
            say Please enter password below if needed.
        endif
        echo ; echo ; echo
        sudo installer -verbose -pkg R-${R_ver}.pkg -target /

        ## check to see if it failed
        if ( $status != 0 ) then
            echo ; echo "R installation failed.  Do you have 'sudo' powers?"
            echo "Please install R manually and rerun this program."
            echo ; echo "https://cran.r-project.org/bin/macosx" ; echo
            \rm R-${R_ver}.pkg
            goto CLEAN_BAD
        endif
        \rm R-${R_ver}.pkg
    else
        echo "Download failed! Please install R manually and rerun this program."
        echo ; echo "https://cran.r-project.org/bin/macosx" ; echo
        goto CLEAN_BAD
    endif
else
    echo ; echo R is installed at $R_bin
    echo Skipping R installation. ; echo
endif   ## end R check

###########################################################################
## download afni and install
echo ; echo Installing afni into $bindir ; echo
curl -O "${afni_link}"

## check for local download of afni
if ( $localPkg != "" )  then
    if ( -f "${localPkg}" ) then
        tcsh @update.afni.binaries -defaults -bindir ${bindir} \
                                   -local_package ${localPkg}
    else
        echo "ERROR: ${localPkg} not found!" ; echo
        goto CLEAN_BAD
    endif
else    ## just download it
    tcsh @update.afni.binaries -defaults -bindir ${bindir} -package ${afni_ver}
endif
set path = ( $path  ${bindir}  )

## afni R packages
echo ; echo install afni R packages.
if ( $loud ) then
    say Please enter password below if needed.
endif
sudo rPkgsInstall -pkgs ALL

## check to see if it failed
if ( $status != 0 ) then
    echo ; echo "rPkgsInstall failed.  Do you have 'sudo' powers?" ; echo
    echo "If you don't, you can try: rPkgsInstall -pkgs ALL"
    echo "Or you can get your admin to run: sudo rPkgsInstall -pkgs ALL"
    goto CLEAN_BAD
endif

###########################################################################
## extras for paths

## xquartz stuff
set csh_add = 'setenv DYLD_LIBRARY_PATH /opt/X11/lib/flat_namespace'
set bash_add = 'export DYLD_LIBRARY_PATH=$DYLD_LIBRARY_PATH:/opt/X11/lib/flat_namespace'

touch ~/.cshrc
set temp_grep = `grep -c "${csh_add}" ~/.cshrc`
if ( $temp_grep == 0 ) then
    echo ${csh_add} >> ~/.cshrc
endif
touch ~/.bashrc
set temp_grep = `grep -c "${bash_add}" ~/.bashrc`
if ( $temp_grep == 0 ) then
    echo ${bash_add} >> ~/.bashrc
endif

## the @update.afni.binaries doesn't do the .bash_profile
touch ~/.bash_profile
set temp_grep = `grep -c "${bindir}" ~/.bash_profile`
if ( $temp_grep == 0 ) then
    echo export 'PATH=$PATH:'${bindir} >> ~/.bash_profile
    echo 'ahdir=`apsearch -afni_help_dir`' >> ~/.bash_profile
    echo 'if [ -f "$ahdir/all_progs.COMP.bash" ]' >> ~/.bash_profile
    echo "then" >> ~/.bash_profile
    echo '   . $ahdir/all_progs.COMP.bash' >> ~/.bash_profile
    echo "fi" >> ~/.bash_profile
endif
set temp_grep = `grep -c "${bash_add}" ~/.bash_profile`
if ( $temp_grep == 0 ) then
    echo ${bash_add} >> ~/.bash_profile
endif

## afnirc etc
if ( ! -f "$HOME/.afnirc" ) then
    \cp ${bindir}/AFNI.afnirc $HOME/.afnirc
endif
if ( ! -f "$HOME/.sumarc" ) then
    suma -update_env
endif

###########################################################################
## done and clean up

# echo ; echo "Seems like everything installed correctly."
echo ; echo "If you've reached this far, then you are *almost* there!"
echo "In order to complete, please:"
echo "   1. Restart your computer (at least logout and in)."
echo "   2. Then run the system check, typing in the terminal:"
echo "          afni_system_check.py -check_all"
echo ; echo "You may want the bootcamp data and Fink from here:"
echo "https://afni.nimh.nih.gov/pub/dist/doc/htmldoc/background_install/install_instructs/steps_mac.html"
echo ; echo "Thank you and goodbye!" ; echo

if ( $loud ) then
    say Seems like everything installed correctly
    say Thank you and goodbye!
endif

\rm -r ${inst_dir}

exit 0

###########################################################################
CLEAN_BAD:
    echo ; echo "Cleaning up!"
    if ( -d "/Volumes/XQuartz-${XQ_ver}" ) then
        hdiutil unmount -force "/Volumes/XQuartz-${XQ_ver}"
    endif
    \rm -rfv ${inst_dir}
    echo
    exit 1

###########################################################################
SHOW_HELP:
cat << EOF

   ----------------------------------------------------------------------------
   $prog
    Helper script to install AFNI and related packages on  macOS .
    In most cases you should just double click the icon for this script.

   ----------------------------------------------------------------------------
    Here is the README.afni_macOS_install_BETA if you lost it:

    Welcome to the afni installation for  macOS !
    You should launch the script by double clicking the icon like a regular program.
    If you get a popup notification about an "unknown developer",
    you can do the following:
       Right click or control click the app icon.
       Then click open from the shortcut menu.
       (see here for more info: https://support.apple.com/kb/PH25088)

    This will launch your terminal.
    Most of the rest of the interactions are in the terminal.
    If needed, the Xcode command line tools will popup an installation window.
    Once complete, return to the terminal to continue.

    To accept the default options (shown in [ ]), just press Enter.
    For some sections, you may need to type in your password.
    If you are not an administrator on your computer, this installation may fail.

    This will download and install the recommended versions of the following:
       1. Xcode command line tools
       2. Xquartz
       3. R
       4. AFNI

    If a package is already installed, you may be asked if you want to reinstall it.
    But you can skip it if you like.

    This script will also setup your .cshrc, .bashrc, and .bash_profile.
    It will add the appropriate entries to your paths specified there.
    After completion, you should restart your computer just in case.

    This script does not install fink. Nor does it download the class data.
    But it will point you to where you can get it.
    (https://afni.nimh.nih.gov/pub/dist/doc/htmldoc/background_install/install_instructs/steps_mac.html)

    Thanks for testing this installation script and good luck.

   -----------------------------------------------------------------------------
   options:

      -local_package : Install from a local copy of an AFNI tgz.
                       Usually from https://afni.nimh.nih.gov/pub/dist/tgz/
                       Make sure it is the correct version!

     -loud           : Make your mac talk to you along the way.
                       Should work, but your results may vary.

      -help          : Show this help.

   -----------------------------------------------------------------------------
   example:

   $prog -local_package ~/Downloads/macosx_10.7_local.tgz

   -----------------------------------------------------------------------------
   Justin Rajendra 09/2017

EOF

exit 0
